;add:

;- game over
;- show next block



			DEVICE ZXSPECTRUM48
pictureBuffer EQU  $7000
LeftTop		  EQU 22535; set to left edge and top line

	ORG 38000

Start	
	EI
	XOR A
	LD (23624),A ;border system variable
	JP GameMenu
	
NewGame
	LD A,1
	LD (LevelNr),A
	
NewLevel	
	LD SP,65530
	CALL ClearScreen
	LD A,(LevelNr)
	CALL UnzipPicture ;unzips picture into memory buffer
	CALL DisplayLevelHeading ;introduction
	CALL DrawLevel
	CALL SetLevelData
	CALL CreateNextBlock
	
;----------------MAIN GAME LOOP----------------------	
MainLoop
	HALT
	
	;check if falling block exists
	LD A,(CurrentBlockNr)
	OR A
	CALL Z, NewBlock; if it doesn't exist, the next block will become it
	
	CALL ReadPlayerInput;keyboard/joystick input
	CALL Action; perfrom action when ready
	
	;decrease time left to fall
	LD HL,CurrentTime
	DEC (HL)
	LD A,(HL)
	OR A ;did time reach 0?
	CALL Z,FallDown

	JR MainLoop
	
;------------------------
SetLevelData
	;find level time
	LD HL,LevelNr
	LD B,(HL)
	LD DE,LevelTimes-1
	
FindLevelData
	INC DE
	DJNZ FindLevelData
	
	LD A,(DE);read time
	INC HL
	LD (HL),A; write level time
	INC HL
	LD (HL),A;write copy of level time
	
	XOR A
	LD (CurrentBlockNr),A ; no falling block at the beginning
	LD (KeyPressed),A
	LD (LineFull),A
	LD (AnyLineFull),A
	
	RET

;-----------------------
NewBlock
	;next block becames current falling block when the game begins or when the previous block reached the bottom
	LD HL,CurrentBlockNr
	LD A,(NextBlockNr)
	LD (HL),A
				
	;rotation 0
	INC HL
	XOR A
	LD (HL),A
	
	;position - top line, centered
	INC HL	
	LD A,4
	LD (HL),A;x
	INC HL
	XOR A
	LD (HL),A;y
	
	
	CALL CheckSpace;check for game over if there isn't any place for new block
	EX AF,AF'
	PUSH AF;save alternate A which stores info if placing a new block is possible
	EX AF,AF'
	
	CALL DrawBlock
	
	POP AF
	OR A
	JP NZ,GameOver
	
	LD A,(LevelTime)
	LD (CurrentTime),A
	XOR A
	LD (KeyPressed),A;reset keyboard input
	
	CALL CreateNextBlock; now when next block became current we need next next block:)
	RET
;--------
CreateNextBlock
	;choose randomly type of block
	CALL RandomGenerator ;random 0-15 in A
	;LD A,14;!!!temp
	LD (NextBlockNr),A
	;draw next block
	CALL DrawNextBlock
	
	RET

;--------
RandomGenerator
;taken from www

Rand16
	LD DE,1111
	LD	A,D
	LD	H,E
	LD	L,253
	OR	A
	SBC	HL,DE
	SBC	A,0
	SBC	HL,DE
	LD	D,0
	SBC	A,D
	LD	E,A
	SBC	HL,DE
	JR	NC,Rand
	INC	HL
Rand
	LD	(Rand16+1),HL; random in hl
	
	LD A,H
	XOR L;
	AND 00001111b; random 0-15 in A
	INC A;1-16
	RET	
	
;------------------------
FallDown
	CALL ClearBlock
	
	LD HL,CurrentBlockY
	INC (HL);go one line down (change position for checking if move is legal)
	CALL CheckSpace; Check if new position is legal
	
	;test alternate A which after CheckSpace holds info if new position is legal
	EX AF,AF'
	LD B,A 
	EX AF,AF'
	LD A,B
	OR A
	JR NZ,FallDownImpossible	
	JR FallDownPossible
	
FallDownImpossible
	LD HL,CurrentBlockY
	DEC (HL);restore block at last position as it doesn't go to new position
	CALL DrawBlock
	
	XOR A
	LD (CurrentBlockNr),A ;this blocks doesn't exist and doesn't move anymore
	
	CALL FallDownSound
	;maybe quick falling was chosen so restore normal speed
	LD A,(LevelTimeCopy)
	LD (LevelTime),A
	
	CALL ClearLines; maybe  there are some full lines, if so clear them
	
	RET	
	
FallDownPossible
	CALL DrawBlock; draw block in new position
	
	;restore time
	LD A,(LevelTime)
	LD (CurrentTime),A
	RET

;------------------------------------
ClearLines
	;go through all the lines 
	LD HL,LeftTop+1; start from first char of game area (LeftTop is edge)
	LD B,22; number of lines on the screen
	XOR A
	LD (AnyLineFull),A; at beginning there aren't any full lines
	
	
;check next line
ClearLines1	
	LD C,10; number of chars in a line
	LD A,1
	LD (LineFull),A ;at beginning of a new line suppose it is full
	
ClearLines2
	LD A,(HL)
	OR A; is empty?
	JR NZ,ClearLines3 ; not empty
	
	XOR A
	LD (LineFull),A; if we reach this code it means line is somewhere empty
ClearLines3
	INC HL
	DEC C
	JR NZ, ClearLines2
	DEC HL; one too many increments
	
	;if line is full highlight it
	LD A,(LineFull)
	OR A
	JR Z,ClearLines4;not full	
	
	;highlight line
	LD A,199;attribute
	LD (AnyLineFull),A; there are full lines (not 0)
	LD D,10
HighlightLine
		
	LD (HL),A
	DEC HL
	DEC D
	JR NZ,HighlightLine	
	LD DE,10
	ADD HL,DE; back to end of line
	
ClearLines4
	LD DE,23
	ADD HL,DE ;beginning of next line
	DJNZ ClearLines1
	
;end of checking and highlighting lines
;if no lines are full just exit
	LD A,(AnyLineFull)
	OR A
	RET Z

;if we reach this code, we know there are full lines
	CALL ClearLineSound ;sound effect


;remove full line and move everything above it one line down
ClearLines5	
	LD HL,LeftTop+1
	LD DE,32
	
;search for highlighted line
	LD B,0; b-nr of checked line, start from top line o
ClearLines6
	LD A,(HL)
	CP 199
	JR Z,ClearLines7;line found
	INC B
	ADD HL,DE
	
	LD A,B
	CP 23; end of screen reached
	RET Z; if so, it means all lines are removed and we can exit
	JR ClearLines6

ClearLines7
	CALL MoveDown;moves everything about cleared line down
	
	LD HL,LinesLeft
	DEC (HL)
	CALL DisplayLinesLeft ;decrease number of lines left to be done
	
	CALL ShowPictureLine; reveal part of level picture
	
	JR ClearLines5; there may be more than one full line so repeat until remove them all
		
	
LineFull 		.BYTE  1
AnyLineFull		.BYTE  0

;-------
MoveDown
;in HL we have adress of line which has to be removed
;we move all the lines above it one line down
;the topmost line is filled with empty spaces

;check if HL is topmost line
;if so fill it with empty space and return
	LD DE,LeftTop+1
	LD A,D
	CP H
	JR NZ,MoveDown2; not topmost
	LD A,E
	CP L
	JR NZ,MoveDown2; not topmost
	
;if we reach this code it means we are at topmost line
	LD B,10
	XOR A
MoveDown1
	LD (HL),A; fill with 0
	INC HL
	DJNZ MoveDown1
	RET; moving lines down finished

	

MoveDown2
;count address of line above current line
	PUSH HL
	INC HL
	LD DE,32
	SBC HL,DE
	EX DE,HL
	POP HL ;current in HL, above in DE
	
;squares from above go down
	LD B,10
MoveDown3
	LD A,(DE)
	LD (HL),A
	INC DE
	INC HL
	DJNZ MoveDown3
	
;go one line up	
	LD DE,42;hl was incread by 11 so we substract 32+11=43
	SBC HL,DE
	JR MoveDown; move next line down




;------------PLAYER INPUT-------------
ReadPlayerInput
	LD A,(KeyPressed); check if some action was already chosen and now waiting for effect
	OR A
	RET NZ; if action was alreade chosen, don't read the keyboard until it is performed
		
;read input
	;keyboard or joystick
	LD A,(PlayerControl)
	OR A

	JR Z,Keyboard
	JR NZ, Joystick

	;after scanning keyboard or oystick program returns here
	;now player action is in A
ReadPlayerInput2	
	OR A
	RET Z; if A is 0 don't do anything
	
	
	LD (KeyPressed),A;store chosen action
	LD A,4
	LD (WaitForAction),A;set time left to perform action
	
	RET

;---
Joystick
	LD BC,31
	IN A,(C)
	JR ReadPlayerInput2	
;---
Keyboard
	;the aim is to set bits of A as it would be Kempston joystick
	LD D,0
	LD BC,57342; o,p	
	IN A,(C)
	CPL
	AND 00000011b;
	LD D,A
	LD BC,64510;q
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	OR D		
	LD D,A		
	LD BC,65022;a
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	OR D
	LD D,A
	LD BC,32766
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	RLA
	OR D ;bit set Ready
	JR ReadPlayerInput2	
	
;------------------------------
Action
	;peforms (when it is possible)  action chosen by player
	
	LD HL,WaitForAction
	DEC (HL)
	RET NZ; if waiting time didn't reach 0 return
	
	LD A,(KeyPressed)
	
	OR A; do nothing?
	RET Z
	
	BIT 0,A;right?
	JP NZ,GoRight
	
	BIT 1,A;left?
	JP NZ,GoLeft
	
	BIT 2,A;drop?
	JP NZ,Drop
	
	BIT 3,A;rotate?
	JP NZ,Rotate
		
	BIT 4,A
	JP NZ,Pause
	RET
	
;-----


GoLeft
	;CALL ShowPictureLine
	
	CALL ClearBlock
	
	LD HL,CurrentBlockX
	
	LD A,(HL)
	OR A;check if block is at left edge
	JR Z,GoLeftPossible; if it is, it cannot go left more, so just redraw it
	
	DEC (HL);go left  (change position for checking if move is legal)
	CALL CheckSpace; Check if new position is legal
	
	;test alternate A which after CheckSpace holds info if new position is legal
	EX AF,AF'
	LD B,A 
	EX AF,AF'
	LD A,B
	OR A
	JR NZ,GoLeftImpossible	
	JR GoLeftPossible
	
GoLeftImpossible
	LD HL,CurrentBlockX
	INC (HL);restore block at last position as it doesn't go to new position
	CALL DrawBlock
	
	XOR A	
	LD(KeyPressed),A;again ready to read input
	RET
	
GoLeftPossible
	CALL DrawBlock; draw block in new position
	XOR A	
	LD(KeyPressed),A;again ready to read input
	RET
	
	
;-----	
GoRight
	CALL ClearBlock
	
	LD HL,CurrentBlockX
	INC (HL);go right  (change position for checking if move is legal)
	CALL CheckSpace; Check if new position is legal
	
	;test alternate A which after CheckSpace holds info if new position is legal
	EX AF,AF'
	LD B,A 
	EX AF,AF'
	LD A,B
	OR A
	JR NZ,GoRightImpossible	
	JR GoRightPossible
	
GoRightImpossible
	LD HL,CurrentBlockX
	DEC (HL);restore block at last position as it doesn't go to new position
	CALL DrawBlock
	
	XOR A	
	LD(KeyPressed),A;again ready to read input
	RET
	
GoRightPossible
	CALL DrawBlock; draw block in new position
	XOR A	
	LD(KeyPressed),A;again ready to read input
	RET

	
;-----
Drop
	XOR A
	LD(KeyPressed),A;again ready to read input
	
	LD A,2
	LD (LevelTime),A; set short falling time
	RET
;-----
Pause
;pauses game
	XOR A
	LD(KeyPressed),A;again ready to read input

;small delay
	LD B,20
Pause1
	HALT
	DJNZ Pause1

;goes in a loop until a key is pressed
Pause2
	LD BC,31;joystick
	IN A,(C)
	BIT 4,A
	JR NZ,Pause3
	
	LD BC,32766;space
	IN A,(C)
	BIT 0,A
	JR Z, Pause3
	JR Pause2

Pause3
;small delay
	LD B,10
Pause4
	HALT
	DJNZ Pause4
	RET

;---------------

Rotate
	CALL ClearBlock
	
	LD HL,CurrentRotation
	LD A,(HL)
	INC A ;new rotation shape
	AND 00000011b; rotation may be from 0 to 3
	LD (HL),A
	CALL CheckSpace; Check if new position is legal
	
	;test alternate A which after CheckSpace holds info if new position is legal
	EX AF,AF'
	LD B,A 
	EX AF,AF'
	LD A,B
	OR A
	JR NZ,RotateImpossible	
	JR RotatePossible
	
RotateImpossible
	LD HL,CurrentRotation
	LD A, (HL)
	DEC A
	AND 00000011b
	LD (HL),A
	CALL DrawBlock;restore block at previous rotation as it is impossible to rotate
	
	XOR A	
	LD(KeyPressed),A;again ready to read input
	RET
	
RotatePossible
	CALL DrawBlock; draw block in new position
	XOR A	
	LD(KeyPressed),A;again ready to read input
	RET
	
;-----sounds------
ClearLineSound
		;play sound
Tweet1
	DI
	LD B,255
	LD DE,1
	LD HL,100
Tweet2	
	PUSH BC
	PUSH DE
	PUSH HL
	
	CALL 949
	
	POP HL
	POP DE
	POP BC
	INC HL
	DJNZ Tweet2
	EI
	RET
;--------
FallDownSound

	PUSH BC
	PUSH DE
	LD E,180				
FallDownSound2
	XOR A
	OUT (254),A
	LD B,E
FallDownSound3
	DEC B
	JR NZ, FallDownSound3
	SET 4,A
	OUT (254),A
	DEC E
	JR NZ, FallDownSound2

	POP DE
	POP BC
	RET
;-----
LevelEndSound

	LD BC,64004
	LD HL,512
	LD DE ,2
LevelEndSound2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL 949
	POP BC
	POP DE
	POP HL
	LD A,L
	SUB C
	LD L,A
	DJNZ LevelEndSound2
	RET


	
;-----
LevelTimes
	.BYTE 14
	.BYTE 13
	.BYTE 12
	.BYTE 11
	.BYTE 10
	.BYTE 9
	.BYTE 8
	.BYTE 7
	.BYTE 6
;-----------------------


CurrentBlockNr	.BYTE 1
CurrentRotation .BYTE 0
CurrentBlockX	.BYTE 0
CurrentBlockY	.BYTE 0

NextBlockNr		.BYTE 0

LevelNr			.BYTE 9
LevelTime		.BYTE 0; how many 1/50s of seconds needs a block to fall one line down in current level
LevelTimeCopy	.BYTE 0
CurrentTime		.BYTE 0; how many time has left now to fall down for a block	
LinesLeft		.BYTE 24

PlayerControl	.BYTE 1; 0-keyboard 1 - Kempton joystick
WaitForAction	.BYTE 0; cycles between pressing key and a result
KeyPressed		.BYTE 0; remembers which key was pressed by player

;-----------------------
	
	INCLUDE <Pictures.asm>
	INCLUDE <Text.asm>
	INCLUDE <Blocks.asm>
	SAVESNA "Mockatetris.sna",Start

