;-------------------------------
DrawBlock
;draws using attributes current falling block
	;first some calculations common for drawing and clearing
	CALL FindBlockData

	;now we can draw the block
	LD DE,29; bytes to go to next line (32-3)
	LD BC,3*256+3 ; block size
	

DrawBlock1
	LD A,(IX+0); read block data
	OR A; filled or empty?
	JR Z,DrawBlock2; if empty skip drawing
	
	;drawing
	EX AF,AF'; color attribute
	LD (HL),A;draw
	EX AF,AF'
	
DrawBlock2
	INC IX
	INC HL
	DEC C
	JR NZ,DrawBlock1
	
	;go to new line
	ADD HL,DE
	LD C,3;again draw three squares in a line
	DJNZ DrawBlock1
		
	RET
;--------------
ClearBlock
	;first some calculations common for drawing and clearing
	CALL FindBlockData

	;now we can clear the block
	LD DE,29; bytes to go to next line (32-3)
	LD BC,3*256+3 ; block size
	

ClearBlock1
	LD A,(IX+0); read block data
	OR A; filled or empty?
	JR Z,ClearBlock2; if empty skip drawing
	
	;drawing
	XOR A
	LD (HL),A;draw
	
ClearBlock2
	INC IX
	INC HL
	DEC C
	JR NZ,ClearBlock1
	
	;go to new line
	ADD HL,DE
	LD C,3;again draw three squares in a line
	DJNZ ClearBlock1
		
	RET

;-------
FindBlockData
	
;read block color
	LD BC,CurrentBlockNr
	LD A,(BC)
	LD HL,BlockColors-1
FindBlockColor
	INC HL
	DEC A
	JR NZ,FindBlockColor	
	
	EX AF,AF'
	LD A,(HL); color in alternate register
	EX AF,AF'
	LD A,(BC);restore block nr
	
;now find beginning of our block data
	LD HL,Blocks-36-9
	LD DE,36
	
	
FindBlock1
	ADD HL,DE
	DEC A
	JR NZ, FindBlock1
	
;now find the block data with curent rotation
	LD DE,9
	INC BC; go to rotation data
	LD A,(BC)
	INC A; rotation 0 means we execute loop which finds rotated data once, so increment

FindBlock2
	ADD HL,DE
	DEC A
	JR NZ, FindBlock2
	PUSH HL
	POP IX
	;beginning of block data now in IX
	
;calculate screen address of block
	;read positon X and Y 
	INC BC; go to x
	LD A,(BC)
	LD D,A
	INC BC; go to y
	LD A,(BC)
	LD E,A
	PUSH DE
	POP BC; x=b,y=c
	INC C ; rotation 0 means we execute loop which find line once, so increment
	
	;transform to HL position
	LD HL,LeftTop-32
	LD DE,32

FindBlockPosition1
	ADD HL,DE
	DEC C
	JR NZ,FindBlockPosition1 ; line done
	
	LD D,0
	LD E,B
	ADD HL,DE ; collumn done
	
	;now we have:
	; -screen position in HL
	; -start of block data in IX
	; -block color in alternate A
	RET
	
;---------------------
DrawNextBlock
;draws in menu picture of next block under text "next block"

;find next block data
	LD BC,NextBlockNr
	LD A,(BC)
	LD HL,BlockColors-1
FindNextBlockColor
	INC HL
	DEC A
	JR NZ,FindNextBlockColor	
	
	EX AF,AF'
	LD A,(HL); color in alternate register
	EX AF,AF'
	LD A,(BC);restore block nr
	
;now find beginning of our block data
;we display it with rotation=0
;so we don't need to include rotation in calculations
	LD HL,Blocks-36
	LD DE,36
	
FindNextBlock1
	ADD HL,DE
	DEC A
	JR NZ, FindNextBlock1
	PUSH HL
	POP IX ;next block data address in IX
	
;now we can draw the block
	LD HL,23060;position
	LD DE,29; bytes to go to next line (32-3)
	LD BC,3*256+3 ; block size
	
DrawNextBlock1
	LD A,(IX+0); read block data
	OR A; filled or empty?
	JR Z,DrawEmpty; if empty erase the square
	

DrawFilled
	EX AF,AF'; color attribute
	LD (HL),A;draw
	EX AF,AF'
	JR DrawNextBlock2
DrawEmpty
	XOR A
	LD (HL),A
	
DrawNextBlock2
	INC IX
	INC HL
	DEC C
	JR NZ,DrawNextBlock1
	
	;go to new line
	ADD HL,DE
	LD C,3;again draw three squares in a line
	DJNZ DrawNextBlock1
	
	RET
	
;-------------------------------------
CheckSpace
;check if at tested position
;we can fit  chosen block with chosen rotation
	
	;prepare in registers data about block new supposed position and about block itself
	CALL FindBlockData
	
	;now we should check if at  new location squares that are covered by shape of our block are empty
	;all of them are legal=location is posible
	;one or more isn't empty=location is forbidden
	
	LD BC,3*256+3
	EX AF,AF'
	XOR A ;0 in alternate A means that location is possible
	EX AF,AF'
	 
CheckSpace1
	
	;forbidden location is when at the given square
	;both block and background aren't empty
	;other cases are legal
	LD D,0
;background	
	LD A,(HL)
	OR A
	JR Z,CheckSpace2 ;do nothing background empty
	INC D;increase D if background not empty
CheckSpace2
;block
	LD A,(IX+0)
	OR A
	JR Z,CheckSpace3 ;do nothing block empty
	INC D;increase D if block not empty

CheckSpace3
	;check if both background and block not empty
	LD A,D
	CP 2; if both not empty D=2
	JR NZ,CheckSpace4;position legal so far
	
	;mark that position is forbidden
	EX AF,AF'
	LD A,255
	EX AF,AF'
	
CheckSpace4
	;go to next square in the line
	INC HL
	INC IX
	DEC C
	JR NZ,CheckSpace1
	
	;go to next line
	LD DE,29
	ADD HL,DE
	LD C,3; again 3 squares in the new line
	DJNZ CheckSpace1
	
	RET

;-------------------------------------
;unzips picture into memory buffer
;A contains number of picture
UnzipPicture
	LD HL, PicAddr-2

;go to record in table containing address of picture which number is stored in A
FindPictureAddr
	INC HL
	INC HL
	DEC A
	JR NZ, FindPictureAddr

;load HL with found address (data source)
	LD E,(HL)
	INC HL
	LD D,(HL)
	EX DE,HL
	
	LD DE,pictureBuffer ;destination
	
	

	CALL DEC40 ;calls depacker
	RET
;--------------------------
CopyPicToScreen
	LD HL,pictureBuffer
	LD DE,16384
	LD BC,6912
	LDIR
	RET
	
;------------------------
ShowPictureLine
	;LD HL,LinesLeft;!!!temp
	;DEC (HL);!!!temp
	;count which line is to show
	LD A,(LinesLeft)
	LD B,A
	LD A,24
	SUB B; nr of picture line in A
	
	;find screen address of our line
	LD B,A
	LD HL,16384-1792-32
	
	
ShowPictureLine1
	LD A,L
	ADC A,32
	LD L,A
	JR NC,ShowPictureLine2
	
	;new screen segment
	LD A,H
	ADD A,8
	LD H,A
ShowPictureLine2
	DJNZ ShowPictureLine1
	
	;line found
	LD A,H
	ADD A,48
	LD D,A
	LD E,L; our line in memory buffer
	
	PUSH HL;draw left side
	PUSH DE
	CALL ShowPictureLineOneSide
	POP DE
	POP HL
	
	LD A,L ;draw right side
	ADD A,25
	LD L,A
	LD E,A
	CALL ShowPictureLineOneSide
	
	;check if all lines are cleared
	LD A,(LinesLeft)
	OR A
	JR Z,ShowLevelScreen
	
	RET
	
;---------
ShowPictureLineOneSide
;pixels
	PUSH HL	
	LD C,8
ShowPictureLine3	
	LD B,7
;one line of bytes	
ShowPictureLine4
	LD A,(DE)
	LD (HL),A;copy from buffer to screen
	INC L
	INC E
	DJNZ ShowPictureLine4
	
	LD A,L
	SUB 7
	LD L,A
	LD E,A;got to beginning of line
	
	INC H
	INC D;go to new byte line
	DEC C
	JR NZ,ShowPictureLine3
		
;attributes
	
	POP HL
	LD A,80
	SRL H;find screen attr address
	SRL H
	SRL H
	ADD A,H
	LD H,A
	
	LD A,H ;find buffer attr address
	ADD A,48
	LD D,A
	LD E,L

;copy attr form buffer to screen	
	
	LD B,7
ShowPictureLine5
	LD A,(DE)
	LD (HL),A
	INC L
	INC E
	DJNZ ShowPictureLine5
	
	RET
	
;-----------------
ShowLevelScreen
	CALL LevelEndSound
	CALL LevelEndSound
	CALL LevelEndSound
	
	CALL CopyPicToScreen

;some delay so player doesn't presses key by chance and erases the picture
	LD B,100
ShowLevelScreen2
	HALT
	DJNZ ShowLevelScreen2
	
;scan keyboard for space and joystick for fire
ShowLevelScreen3
	LD BC,31;joystick
	IN A,(C)
	BIT 4,A
	JR NZ,ShowLevelScreen4
	
	LD BC,32766;space
	IN A,(C)
	BIT 0,A
	JR Z,ShowLevelScreen4
	
	JR ShowLevelScreen3
	
ShowLevelScreen4
	LD HL,LevelNr
	LD A,(HL)
	INC A
	CP 10; end of game, all levels done?
	JP Z,GameWin
	
	;go to next level
	LD (HL),A
	JP NewLevel
	

;--------------------------
;depacking routine
DEC40
        LD      A,#80
        EX      AF,AF'
MS      LDI
M0      LD      BC,#2FF
M1      EX      AF,AF'
M1X     ADD     A,A
        JR      NZ,M2
        LD      A,(HL)
        INC     HL
        RLA
M2      RL      C
        JR      NC,M1X
        EX      AF,AF'
        DJNZ    X2
        LD      A,2
        SRA     C
        JR      C,N1
        INC     A
        INC     C
        JR      Z,N2
        LD      BC,#33F
        JR      M1

X2      DJNZ    X3
        SRL     C
        JR      C,MS
        INC     B
        JR      M1
X6
        ADD     A,C
N2
        LD      BC,#4FF
        JR      M1
N1
        INC     C
        JR      NZ,M4
        EX      AF,AF'
        INC     B
N5      RR      C
        RET     C
        RL      B
        ADD     A,A
        JR      NZ,N6
        LD      A,(HL)
        INC     HL
        RLA
N6      JR      NC,N5
        EX      AF,AF'
        ADD     A,B
        LD      B,6
        JR      M1
X3
        DJNZ    X4
        LD      A,1
        JR      M3
X4      DJNZ    X5
        INC     C
        JR      NZ,M4
        LD      BC,#51F
        JR      M1
X5
        DJNZ    X6
        LD      B,C
M4      LD      C,(HL)
        INC     HL
M3      DEC     B
        PUSH    HL
        LD      L,C
        LD      H,B
        ADD     HL,DE
        LD      C,A
        LD      B,0
        LDIR
        POP     HL
        JR      M0
END_DEC40
	RET	

;-------------pictures--------------       	

;table with memory addresses of packed pictures
PicAddr
	WORD Pic1
	WORD Pic2
	WORD Pic3
	WORD Pic4
	WORD Pic5
	WORD Pic6
	WORD Pic7
	WORD Pic8
	WORD Pic9


;files with packed pictures included
Pic1
	INCBIN "Sotb2.mlz"
Pic2
	INCBIN "Wof.mlz"
Pic3
	INCBIN "Ishar.mlz"
Pic4
	INCBIN "LostVikings.mlz"
Pic5
	INCBIN "Megalomania.mlz"
Pic6
	INCBIN "Blackthorne.mlz"
Pic7
	INCBIN "Trojan.mlz"
Pic8
	INCBIN "ArcticMoves.mlz"
Pic9
	INCBIN "Caveman.mlz"

;-----------Drawing routines----------
ClearScreen
	XOR A
	OUT (254),A; border
	
	LD HL,16384
	LD (HL),A
	LD DE,16385
	LD BC,6911
	LDIR
	RET

DrawLevel
	CALL ClearScreen
	
;draw graphical elements	
	LD HL,16384
	LD DE,Chessboard
	LD BC,32*256+24
	CALL TileBlock
	
	LD HL,16391
	LD DE,Edge
	LD BC,256+22
	CALL TileBlock
	
	LD HL,16402
	LD DE,Edge
	LD BC,256+22
	CALL TileBlock
	
	LD HL,16408
	LD DE,Edge
	LD BC,256+22
	CALL TileBlock
	
	LD HL,16403
	LD DE,Empty
	LD BC,5*256+22
	CALL TileBlock
	
	LD A,24
	LD (16391),A
	LD (16402),A
	LD (16408),A
	
	LD HL,16392
	LD DE,Brick
	LD BC,10*256+22
	CALL TileBlock
	
	LD HL,20500
	LD DE,Brick
	LD BC, 3*256+3
	CALL TileBlock
	
	
	
	LD HL, 20679
	LD DE,Logo
	CALL DrawTile
	
;type texts
	LD HL,16499
	LD DE,LevelText
	CALL TypeText
	
	LD A,24
	LD (LinesLeft),A
	LD HL,16627
	LD DE,LinesText
	CALL TypeText
		
	LD HL,18579
	LD DE,BlockText
	CALL TypeText
	
	LD HL,16565
	CALL DisplayLevelNr
	
	CALL DisplayLinesLeft
	RET

;--------------------------------
TileBlock
;hl-position of left upper corner, bc-block size, de-tile address in memory	
	
TileBl1	PUSH BC
		PUSH DE
		PUSH HL

;draws one horizontal line of tiles
TileBl2		PUSH BC
			PUSH DE
			
			PUSH HL
			CALL DrawTile; draw a single tile
			POP HL
			
			POP DE ; restore screen position and tile data address
	
			LD A,(DE)
			LD B,A ;read tile width from memory  to b
	
			LD A,L
			ADD A,B
			LD L,A; count horizontal coordinate of next tile in the line
	 			  ;(add b=tile size to coordinate of previous tile)
			
			POP BC	;restore block size in bc
			DJNZ TileBl2 ;decrease number of tiles in the line yet to do
		
		
; draw c lines of tiles
	POP HL
	POP DE
	
	INC DE
	LD A,(DE)	;read tile height from memory to C
	LD C,A	
	DEC DE	
	
		LD A,L	
TileBl3	ADD A,32 ;go c lines down,because c=tile height
		DEC C
		JR NZ, TileBl3
		LD L,A
		JR NC,TileBl4
		LD A,H
		ADD A,8;next screen segment
		LD H,A		
		
				
TileBl4	POP BC ;restore block size in bc
		DEC C	;decrease number of columns left to do
		JR NZ, TileBl1
		
		RET


;--------------------
DrawTile:
;hl-position,de-address
;first 2 sprite bytes contain size of the sprite
;bc register is loaded with size

	LD A,(DE)
	LD B,A ; x width
	INC DE
	LD A,(DE)
	LD C,A ; y-height	
	INC DE			
	
DrawTile0				
	PUSH HL ;store screen position
	PUSH BC	;store data adress		
								
;this loop draw "c" lines of chars
DrawTile1	PUSH HL
			PUSH BC
		
;this loop draws one line of chars
DrawTile2					
					PUSH HL
					PUSH BC
					LD B,8			
					
;this loop draws one char
	LD A,(DE)
	CP 16
	JR Z,DrawTile3Empty

DrawTile3Normal			
					LD A, (DE)
					LD (HL),A ;repeated 8 times
					INC H
					INC DE
					DJNZ DrawTile3Normal
					JR DrawTile4

DrawTile3Empty
		XOR A ;fills char square with zeroes
			  ;takes less memory than storing all the zeroes
		INC DE ;goes to next byte of data															
DrawTile3Empty2		
		LD (HL),A
		INC H
		DJNZ DrawTile3Empty2
			  
			  
			  
			  				
DrawTile4		POP BC
				LD A,L
				POP HL ;restores h to address of upper row of the line
				LD L,A
				INC L	;moves one char right 
					 
				DJNZ DrawTile2 ;repeat until whole line of chars is drawn
				
				POP BC
				POP HL		
			; goes to another line of chars in the sprite
			LD A,L
			ADD A,32
			LD L,A
			JR NC,DrawTile5;not carry=no going to next segment, so no h changes needed
				
				;change h only if going to next screen segment	
			LD A,H
			ADD A,8
			LD H,A
					
DrawTile5			
			DEC C ;decrease number of chars line left to do
			JR NZ, DrawTile1
;--------
;Attributes
					
; tile drawing is finished
; now restore tile size into BC and position into HL
;  de is already increased to next byte where attributes data begin	
		POP BC
		POP HL		
;So BC-size,DE-data address, hl-screen position						
		
;transform graphic address to attribute address
TileAttr		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A
			
;draw c lines of attributes
TileAttr1 	PUSH BC
			PUSH HL
		
;draw one line of attributes
TileAttr2   LD A,(DE)
			LD (HL),A
TileAttr3	INC HL ;goes to next square in the row
			INC DE
			DJNZ TileAttr2
	
		POP HL
		PUSH DE ;goes to next row of attributes	
		LD DE,32   
		ADD HL,DE  
		POP DE 
			 
		POP BC
		DEC C ;decrease number of lines left to do 
		JR NZ, TileAttr1
		RET
		
		
;-------Game graphics-------
Chessboard
	.BYTE 1,1
	.BYTE 85,170,85,170,85,170,85,170
	.BYTE 1
Edge
	.BYTE 1,1
	.BYTE 60,60,60,60,60,60,60,60
	.BYTE 70
	
Brick
	.BYTE 1,1
	.BYTE	  0,122,118,108, 90, 52,104,  0
	.BYTE	 0

Empty
	.BYTE 1,1
	.BYTE 16
	.BYTE 71

Logo
	.BYTE 18,2
	.BYTE	 62, 63, 63, 31, 15,  0, 63, 95
	.BYTE	  0,255,255,255,255,  0,255,255
	.BYTE	  0,255,255,255,255,  0,255,255
	.BYTE	  0,255,255,255,255,  0,254,252
	.BYTE	  0,241,229,205,157, 60,124,252
	.BYTE	  0,241,229,205,157, 60,124,252
	.BYTE	  0,255,255,255,255,  0,124,126
	.BYTE	  0,254,254,254,254,  0, 60,124
	.BYTE	  0, 31,223,223,223,192,204,204
	.BYTE	  0,255,128,191,191,  0,252,254
	.BYTE	  0,255,  0,255,255, 96, 99,103
	.BYTE	126,255, 12,235,235,  3,227,243
	.BYTE	  0,255,127, 63,191,128, 14, 30
	.BYTE	  0,255, 15,111,111,  0, 97, 99
	.BYTE	  0,255,255,255,255,  0,247,247
	.BYTE	  0,255,255,255,255,  0,255,255
	.BYTE	  0,255,255,255,255,  0,255,255
	.BYTE	124,252,252,248,240,  0,252,250
	.BYTE	 32, 31,  0,  0,  3,  5,  2,  1
	.BYTE	  0,255,  0,  0,255,255,  0,255
	.BYTE	  0,255,  0,  0,255,255,  0,252
	.BYTE	  1,243,  7, 15,158, 61,122,244
	.BYTE	253,251,247,207,158, 61,122,244
	.BYTE	252,252,252,220,156, 28, 28, 28
	.BYTE	  6,198,198,198,198,198,254,124
	.BYTE	 96, 96, 96, 96, 96, 96,124, 60
	.BYTE	220,248,240,224,240,248,220,204
	.BYTE	  6,126,254,198,198,198,254,124
	.BYTE	102,102,103,103,102,102,103, 99
	.BYTE	 51, 51,243,243,  3, 51,243,225
	.BYTE	 24, 24, 24, 24, 24, 24,152,152
	.BYTE	 99, 99, 99, 97, 96, 96, 99, 99
	.BYTE	  0,  7,224,240, 55, 55,240,231
	.BYTE	  0,255,  0,  0,255,255,  0,255
	.BYTE	  0,255,  0,  0,255,255,  0,255
	.BYTE	  4,248,  0,  0,192,160, 64,128
	.BYTE	 70, 70, 70, 70, 70, 70, 70, 70
	.BYTE	 70, 70, 70, 70, 70, 70, 70, 70
	.BYTE	 70, 70, 70, 70, 70, 70, 70, 70
	.BYTE	 70, 70, 70, 70, 70, 70, 70, 70
	.BYTE	 70, 70, 70, 70
